program OneSeg_Jumper;


uses
  SysUtils,Classes,DateUtils,Windows,INIFiles,Variants,
  XMLDoc,XMLIntf,ActiveX,StdCtrls,ShellAPI,StrUtils;



// ----------------------------
//   TBF
// ----------------------------

function AnalyzeTBF(fname:String; ProgramCh:TStringList; var FromDate,FromTime,ToDate,ToTime:TDateTime;
                    var Channel:Integer; var Subject:String): Boolean;
var
  i          : Integer;
  XMLd       : IXMLDocument;
  XMLn       : IXMLNode;
  td,tim,dat : String;
begin
  Result := False;
  try
    XMLd := LoadXMLDocument(fname);
    XMLn := XMLd.DocumentElement;
    if (XMLn.NodeName='tbf') then
    begin
      Channel := StrToIntDef(ProgramCh.Values[XMLn.ChildNodes.Nodes['ProgramChannel'].NodeValue],-1);
      td      := XMLn.ChildNodes.Nodes['ProgramStartTime'].NodeValue;
      i       := Pos(' ',td);
      tim     := '';
      dat     := '';
      if (i<>0) then
      begin
        dat := Copy(td,1,i-1);
        tim := Copy(td,i+1,Length(td));
      end;
      FromDate := StrToDate(dat);
      FromTime := StrToTime(tim);

      td      := XMLn.ChildNodes.Nodes['ProgramEndTime'].NodeValue;
      i       := Pos(' ',td);
      tim     := '';
      dat     := '';
      if (i<>0) then
      begin
        dat := Copy(td,1,i-1);
        tim := Copy(td,i+1,Length(td));
      end;
      ToDate  := StrToDate(dat);
      ToTime  := StrToTime(tim);
      //
      Subject := XMLn.ChildNodes.Nodes['ProgramTitle'].NodeValue;
      Result  := True;
    end;
  except
    //
  end;
end;


// ----------------------------
//   TVTL
// ----------------------------

function AnalyzeTVTL(fname:String; StationID:TStringList; TVTLTopic:Boolean;
                     var FromDate,FromTime,ToDate,ToTime:TDateTime;
                     var Channel:Integer; var Subject:String): Boolean;
var
  i          : Integer;
  XMLd       : IXMLDocument;
  XMLn,XMLitm: IXMLNode;
  td,tim,dat : String;
begin
  Result := False;
  try
    XMLd := LoadXMLDocument(fname);
    XMLn := XMLd.DocumentElement;
    if (XMLn.NodeName='tvtl') then
    begin
      XMLitm  := XMLn.ChildNodes.Nodes['item'];
      Channel := StrToIntDef(stationID.Values[XMLitm.ChildNodes.Nodes['stationID'].NodeValue],-1);
      if (TVTLtopic) then
      begin
        td    := XMLitm.ChildNodes.Nodes['topicStartDateTime'].NodeValue;
      end
        else
      begin
        td    := XMLitm.ChildNodes.Nodes['programStartDateTime'].NodeValue;
      end;
      i       := Pos('T',td);
      tim     := '';
      dat     := '';
      if (i<>0) then
      begin
        dat := Copy(td,1,i-1);
        tim := Copy(td,i+1,Length(td));
        i   := Pos('.',tim);
        if (i<>0) then tim := Copy(tim,1,i-1);
        i   := Pos('+',tim);
        if (i<>0) then tim := Copy(tim,1,i-1);
      end;
      dat := AnsiReplaceStr(dat,'-','/');
      FromDate := StrToDate(dat);
      FromTime := StrToTime(tim);

      if (TVTLtopic) then
      begin
        td    := XMLitm.ChildNodes.Nodes['topicEndDateTime'].NodeValue;
      end
        else
      begin
        td    := XMLitm.ChildNodes.Nodes['programEndDateTime'].NodeValue;
      end;
      i       := Pos('T',td);
      tim     := '';
      dat     := '';
      if (i<>0) then
      begin
        dat := Copy(td,1,i-1);
        tim := Copy(td,i+1,Length(td));
        i   := Pos('.',tim);
        if (i<>0) then tim := Copy(tim,1,i-1);
        i   := Pos('+',tim);
        if (i<>0) then tim := Copy(tim,1,i-1);
      end;
      dat := AnsiReplaceStr(dat,'-','/');
      ToDate := StrToDate(dat);
      ToTime := StrToTime(tim);
      //
      Subject := XMLitm.ChildNodes.Nodes['programTitle'].NodeValue;
      Result  := True;
    end;
  except
    //
  end;
end;


// ----------------------------
//   iEPG
// ----------------------------

function AnalyzeiEPG(fname:String; StationID:TStringList;
                     var FromDate,FromTime,ToDate,ToTime:TDateTime;
                     var Channel:Integer; var Subject:String): Boolean;
var
  iEPG : TStringList;
  str  : String;
  y,m,d: Integer;
  st,ed: String;
begin
  Result := False;
  try
    iEPG := TStringList.Create;
    try
      iEPG.NameValueSeparator := ':';
      iEPG.LoadFromFile(fname);
      str := Trim(iEPG.Values['Content-type']);
      if (Pos('application/x-tv-program-info',LowerCase(str))<>0) then
      begin
        // iEPG炵
        Channel  := StrToIntDef(stationID.Values[Trim(iEPG.Values['station'])],-1);
        y        := StrToIntDef(iEPG.Values['year'],YearOf(Now));
        m        := StrToIntDef(iEPG.Values['month'],MonthOf(Now));
        d        := StrToIntDef(iEPG.Values['date'],DayOf(Now));
        st       := Trim(iEPG.Values['start']);
        ed       := Trim(iEPG.Values['end']);
        Subject  := Trim(iEPG.Values['program-title']);
        FromDate := EncodeDate(y,m,d);
        ToDate   := FromDate;
        FromTime := StrToTime(st);
        ToTime   := StrToTime(ed);
        if (FromTime>ToTime) then ToDate := IncDay(ToDate,1);
        Result := True;
      end;
    finally
      iEPG.Free;
    end;
  except
  end;
end;


// ----------------------------
//   C
// ----------------------------

var
  INI         : TINIFile;
  ProgramCh   : TStringList;
  StationID   : TStringList;
  DateFmt     : TFormatSettings;
  Channel     : Integer;
  FromDate    : TDateTime;
  FromTime    : TDateTime;
  ToDate      : TDateTime;
  ToTime      : TDateTime;
  Subject     : String;
  ErrorFlg    : Boolean;
  XMLDone     : Boolean;
  FName       : String;
  JumpURL     : String;
  str         : String;
  TVTLtopic   : Boolean;
begin
  // --------------------------------
  //   INIǂݍ
  // --------------------------------
  ProgramCh       := TStringList.Create;
  StationID       := TStringList.Create;
  INI             := TINIFile.Create(IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)))+'OneSeg_Config.ini');
  try
    JumpURL         := INI.ReadString ('Extract','JumpURL'           ,''   );
    TVTLtopic       := INI.ReadBool   ('Extract','TVTLtopic'         ,True );
  finally
    INI.Free;
  end;

  INI             := TINIFile.Create(IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)))+'OneSeg_Channels.ini');
  try
    INI.ReadSectionValues('ProgramChannel',ProgramCh);
    INI.ReadSectionValues('stationID',StationID);
  finally
    INI.Free;
  end;

  // --------------------------------
  //   p[^
  // --------------------------------
  Channel     := 0;
  FromDate    := 0;
  FromTime    := 0;
  ToDate      := 0;
  ToTime      := 0;
  ErrorFlg    := False;

  if (ParamCount = 1) then
  begin
    //   t@Cw胂[h
    // -------------------------------
    FName := ParamStr(1);
    XMLDone := False;
    CoInitialize(nil);
    if FileExists(FName) then
    begin
      if (AnalyzeTBF(FName,ProgramCh,FromDate,FromTime,ToDate,ToTime,Channel,Subject)) then
      begin
        XMLDone := True;
      end
        else
      begin
        if (AnalyzeTVTL(FName,StationID,TVTLTopic,FromDate,FromTime,ToDate,ToTime,Channel,Subject)) then
        begin
          XMLDone := True;
        end
          else
        begin
          if (AnalyzeiEPG(FName,StationID,FromDate,FromTime,ToDate,ToTime,Channel,Subject)) then
          begin
            XMLDone := True;
          end;
        end;
      end;
    end;
    CoUninitialize;

    if (not XMLDone) then
    begin
      MessageBox(0,'Bad tag file','Error',0);
      ErrorFlg := True;
    end;
  end
    else
  begin
    //   G[
    // -------------------------------
    MessageBox(0,'Bad tag file','Error',0);
    ErrorFlg := True;
  end;
  ProgramCh.Free;
  StationID.Free;

  if (ErrorFlg) then halt(1);

  DateFmt.ShortDateFormat := 'yymmdd';
  DateFmt.ShortTimeFormat := 'hhnnss';
  DateFmt.LongTimeFormat := 'hhnnss';
  str := JumpURL+DateToStr(FromDate,DateFmt)+'-'+TimeToStr(FromTime,DateFmt)+'-'+TimeToStr(ToTime,DateFmt)+'-'+IntToStr(Channel);
  ShellExecute( 0, 'open', PChar(str), nil, nil, 0 );
end.

