unit SiUSBXp_h;

interface

uses
  Windows, Messages;

const
  // Return codes
  SI_SUCCESS                = $00;
  SI_DEVICE_NOT_FOUND       = $FF;
  SI_INVALID_HANDLE         = $01;
  SI_READ_ERROR             = $02;
  SI_RX_QUEUE_NOT_READY     = $03;
  SI_WRITE_ERROR            = $04;
  SI_RESET_ERROR            = $05;
  SI_INVALID_PARAMETER      = $06;
  SI_INVALID_REQUEST_LENGTH = $07;
  SI_DEVICE_IO_FAILED       = $08;
  SI_INVALID_BAUDRATE       = $09;
  SI_FUNCTION_NOT_SUPPORTED = $0a;
  SI_GLOBAL_DATA_ERROR      = $0b;
  SI_SYSTEM_ERROR_CODE      = $0c;
  SI_READ_TIMED_OUT         = $0d;
  SI_WRITE_TIMED_OUT        = $0e;
  SI_IO_PENDING             = $0f;

  // GetProductString() function flags
  SI_RETURN_SERIAL_NUMBER   = $00;
  SI_RETURN_DESCRIPTION     = $01;
  SI_RETURN_LINK_NAME       = $02;
  SI_RETURN_VID             = $03;
  SI_RETURN_PID             = $04;

  // RX Queue status flags
  SI_RX_NO_OVERRUN          = $00;
  SI_RX_EMPTY               = $00;
  SI_RX_OVERRUN             = $01;
  SI_RX_READY               = $02;

  // Buffer size limits
  SI_MAX_DEVICE_STRLEN      = 256;
  SI_MAX_READ_SIZE          = 4096*16;
  SI_MAX_WRITE_SIZE         = 4096;

  // Input and Output pin Characteristics
  SI_HELD_INACTIVE          = $00;
  SI_HELD_ACTIVE            = $01;
  SI_FIRMWARE_CONTROLLED    = $02;
  SI_RECEIVE_FLOW_CONTROL   = $02;
  SI_TRANSMIT_ACTIVE_SIGNAL = $03;
  SI_STATUS_INPUT           = $00;
  SI_HANDSHAKE_LINE         = $01;

  // Mask and Latch value bit definitions
  SI_GPIO_0                 = $01;
  SI_GPIO_1                 = $02;
  SI_GPIO_2                 = $04;
  SI_GPIO_3                 = $08;

  // GetDeviceVersion() return codes
  SI_CP2101_VERSION         = $01;
  SI_CP2102_VERSION         = $02;
  SI_CP2103_VERSION         = $03;

  // DLL Name
  SI_DLLNAME                = 'SiUSBXp.dll';

type
  // Type definitions
  SI_STATUS          = integer;
  SI_DEVICE_STRING   = array [0..SI_MAX_DEVICE_STRLEN-1] of char;
  SI_HANDLE          = Pointer;


function SI_GetNumDevices(
var lpdwNumDevices:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;


function SI_GetProductString(
dwDeviceNum:DWORD;
var lpvDeviceString;
dwFlags:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_Open(
dwDevice:DWORD;
var cyHandle:SI_HANDLE
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_Close(
cyHandle:SI_HANDLE
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_Read(
cyHandle:SI_HANDLE;
var lpBuffer;
dwBytesToRead:DWORD;
var lpdwBytesReturned:DWORD;
lpOverlapped: POverlapped
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_Write(
cyHandle:SI_HANDLE;
const lpBuffer;
dwBytesToWrite:DWORD;
var lpdwBytesWritten:DWORD;
lpOverlapped: POverlapped
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_DeviceIOControl(
cyHandle:SI_HANDLE;
dwIoControlCode:DWORD;
lpInBuffer:Pointer;
dwBytesToRead:DWORD;
lpOutBuffer:Pointer;
dwBytesToWrite:DWORD;
var lpdwBytesSucceeded:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_FlushBuffers(
cyHandle :SI_HANDLE;
FlushTransmit:BYTE;
FlushReceive:BYTE
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_SetTimeouts(
dwReadTimeout:DWORD;
dwWriteTimeout:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_GetTimeouts(
var lpdwReadTimeout:DWORD;
var lpdwWriteTimeout:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_CheckRXQueue(
cyHandle:SI_HANDLE;
var lpdwNumBytesInQueue:DWORD;
var lpdwQueueStatus:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_SetBaudRate(
cyHandle:SI_HANDLE;
dwBaudRate:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;


function SI_SetBaudDivisor(
cyHandle:SI_HANDLE;
wBaudDivisor:WORD
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_SetLineControl(
cyHandle :SI_HANDLE;
wLineControl:WORD
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_SetFlowControl(
cyHandle :SI_HANDLE;
bCTS_MaskCode :BYTE;
bRTS_MaskCode :BYTE;
bDTR_MaskCode :BYTE;
bDSR_MaskCode :BYTE;
bDCD_MaskCode :BYTE;
bFlowXonXoff:BOOL
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_GetModemStatus(
cyHandle :SI_HANDLE;
ModemStatus:PBYTE
): SI_STATUS; stdcall; external SI_DLLNAME;

    
function SI_SetBreak(
cyHandle :SI_HANDLE;
wBreakState:WORD
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_ReadLatch(
cyHandle:SI_HANDLE;
var lpbLatch:BYTE
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_WriteLatch(
cyHandle:SI_HANDLE;
bMask:BYTE;
bLatch:BYTE
): SI_STATUS; stdcall; external SI_DLLNAME;


     
function SI_GetPartNumber(
cyHandle:SI_HANDLE;
var lpbPartNum:BYTE
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_GetDLLVersion(
var HighVersion:DWORD;
var LowVersion:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;

     
function SI_GetDriverVersion(
var HighVersion:DWORD;
var LowVersion:DWORD
): SI_STATUS; stdcall; external SI_DLLNAME;




implementation


end.
