unit pMainForm;

{$WARN UNIT_PLATFORM OFF}
{$WARN SYMBOL_PLATFORM OFF}
{$WARN SYMBOL_DEPRECATED OFF}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SiUSBXp_h, StdCtrls, Contnrs, ExtCtrls, ComCtrls, Menus, FileCtrl,
  INIFiles, DateUtils, pDaysForm, Taskbar, StrUtils;

const
  STATE_NONE     = 0;
  STATE_INIT1    = 1;
  STATE_RUNNING  = 2;
  STATE_PENDING  = 9;
  STATE_BUSY     = 10;
  STATE_DISABLE  = 99;
  TUNER_UOT      = 0;
  TUNER_J200     = 1;
  TUNER_NONE     = 99;
  CNF_SEARCHPKT  = 1024;
  WCHDOG_SECS    = 15;
  WCHDOG_ENABLE  = 1;
  WCHDOG_DISABLE = 2;

type
  TOneSegTuner = class(TObject)
    Handle         : SI_HANDLE;
    Number         : Integer;
    TunerID        : String;
    FriendlyName   : String;
    TunerType      : Integer;
    Disable        : Boolean;
    PacketSize     : WORD;
    Channel        : Integer;
    LastTime       : TDateTime;
    CheckFlag      : Boolean;
    TunerState     : Integer;
    WatchDogState  : Integer;
    WatchDogSecs   : Integer;
    StatusText     : String;
    FileName       : String;
    FilePath       : String;
    BeforeFileName : String;
    OutFile        : TFileStream;
    IdxFile        : TFileStream;
    RecvPacket     : LongInt;
    BeforeBuf      : array[0..197] of Char;
    BeforeBufSize  : Integer;
    LastTimeDiv    : TDateTime;
    RetryNum       : Integer;
  end;

  TOneSegForm = class(TForm)
    TunerTab: TTabControl;
    TabPanel: TPanel;
    Label1: TLabel;
    TunerIDEdit: TEdit;
    Label2: TLabel;
    ChannelCombo: TComboBox;
    MainMenu1: TMainMenu;
    F1: TMenuItem;
    X1: TMenuItem;
    R1: TMenuItem;
    R2: TMenuItem;
    TSFolderLabel: TLabel;
    Label3: TLabel;
    StatusEdit: TEdit;
    ProbeTimer: TTimer;
    RecordTimer: TTimer;
    PacketCountEdit: TEdit;
    D1: TMenuItem;
    TrayIcon1: TTrayIcon;
    PopupMenu1: TPopupMenu;
    N1: TMenuItem;
    X2: TMenuItem;
    WindowUpdateTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TunerTabChange(Sender: TObject);
    procedure X1Click(Sender: TObject);
    procedure R2Click(Sender: TObject);
    procedure A1Click(Sender: TObject);
    procedure ChannelComboChange(Sender: TObject);
    procedure ProbeTimerTimer(Sender: TObject);
    procedure RecordTimerTimer(Sender: TObject);
    procedure D1Click(Sender: TObject);
    procedure N1Click(Sender: TObject);
    procedure WindowUpdateTimerTimer(Sender: TObject);
  private
    { Private 錾 }
    ININame       : String;
    FTSFolder     : String;
    FRecDays      : Integer;
    SelectedTuner : TOneSegTuner;
    Tuners        : TObjectList;
    LastDeleteNow : TDateTime;
    ProgramCh     : TStringList;
    StationID     : TStringList;
    //
    function  Tuner_Bootup          (Tuner:TOneSegTuner): Integer;
    function  Tuner_SetChannel      (Tuner:TOneSegTuner; ch:Integer): Integer;
    function  Tuner_Stop            (Tuner:TOneSegTuner): Integer;
    // UOT-100/LOG-J100/LOG-J200
    function  UOTJ200_SetRegister   (Tuner:TOneSegTuner; reg:WORD; dat:BYTE): Integer;
    function  UOTJ200_PlainSend     (Tuner:TOneSegTuner; reg1:BYTE; reg2:BYTE; reg3:BYTE; reg4:BYTE; reg5:BYTE): Integer;
    function  UOTJ200_ReadRegister  (Tuner:TOneSegTuner; reg:WORD; var dat:BYTE): Integer;
    // Control UOT-100 / LOG-J100
    function  UOT_Bootup            (Tuner:TOneSegTuner): Integer;
    function  UOT_SetChannel        (Tuner:TOneSegTuner; ch:Integer): Integer;
    function  UOT_Stop              (Tuner:TOneSegTuner): Integer;
    // Control LOG-J200
    function  J200_Initialize       (Tuner:TOneSegTuner; var dat:BYTE): Integer;
    function  J200_Bootup           (Tuner:TOneSegTuner): Integer;
    function  J200_SetChannel       (Tuner:TOneSegTuner; ch:Integer): Integer;
    function  J200_Stop             (Tuner:TOneSegTuner): Integer;
    //
    function  SerialToTuner      (Serial:String):TOneSegTuner;
    procedure RefreshTunerList;
    procedure RefreshChannelList;
    procedure InitializeInterface(Tuner: TOneSegTuner);
    procedure InitializeTuner    (Tuner: TOneSegTuner);
    procedure DeleteTuner        (Tuner: TOneSegTuner);
    procedure SetTSFolder        (str:String);
    procedure SetRecDays         (i:Integer);
    function  AnalyzeTOT         (buf:PChar; tot:PChar): TDateTime;
    procedure DeleteOldFiles     (nw:TDateTime);
    procedure TunerTabUpdate     (Sender: TObject);
  public
    { Public 錾 }
    property RecDays      : Integer read FRecDays       write SetRecDays;
    property TSFolder     : String  read FTSFolder      write SetTSFolder;
  end;

var
  OneSegForm: TOneSegForm;

implementation

{$R *.dfm}

// ==========================================================
//   
// ==========================================================

procedure TOneSegForm.FormCreate(Sender: TObject);
var
  INI: TINIFile;
begin
  ININame := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName))+'OneSeg_Config.ini';
  inherited;
  // ݒt@C
  INI := TINIFile.Create(ININame);
  try
    TSFolder      := INI.ReadString ('Config','TSFolder','');
    RecDays       := INI.ReadInteger('Config','RecDays',3);
  finally
    INI.Free;
  end;
  // `lf[^
  INI := TINIFile.Create(IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName))+'OneSeg_Channels.ini');
  ProgramCh := TStringList.Create;
  stationID := TStringList.Create;
  try
    INI.ReadSectionValues('ProgramChannel',ProgramCh);
    INI.ReadSectionValues('stationID',StationID);
  finally
    INI.Free;
  end;

  //vZX̗D揇ʂύX
  SetPriorityClass(GetCurrentProcess, REALTIME_PRIORITY_CLASS);
  //Xbh̗D揇ʂύX
  SetThreadPriority(GetCurrentThread, THREAD_PRIORITY_TIME_CRITICAL);

  SelectedTuner := nil;
  Tuners        := TObjectList.Create;
  LastDeleteNow := 0;
  RefreshChannelList;
  RefreshTunerList;
end;


// ==========================================================
//   I
// ==========================================================

procedure TOneSegForm.FormDestroy(Sender: TObject);
var
  i: Integer;
begin
  // Tuner ListSJ
  if Assigned(Tuners) then
  begin
    for i:=0 to Tuners.Count-1 do
    begin
      DeleteTuner(TOneSegTuner(Tuners[i]));
      Sleep(1000);
    end;
    Tuners.Free;
  end;
end;


// ==========================================================
//   Ât@C̍폜
// ==========================================================

procedure TOneSegForm.DeleteOldFiles(nw:TDateTime);
var
  sr1,sr2: TSearchRec;
  y,m,d,h: Integer;
  fexsts : Boolean;
begin
  // fBNg̃XL
  if FindFirst(IncludeTrailingPathDelimiter(TSFolder)+'*.*', faDirectory , sr1) = 0 then
  begin
    repeat
      if (sr1.Attr and faDirectory) <> 0 then
      begin
        y := StrToIntDef(Copy(sr1.Name,1,2),-1);
        m := StrToIntDef(Copy(sr1.Name,3,2),-1);
        d := StrToIntDef(Copy(sr1.Name,5,2),-1);
        if ((Length(sr1.Name)=6) and (y<>-1) and (m<>-1) and (d<>-1)) then
        begin
          // fBNgt炵
          if ((DateOf(nw)-EncodeDate(y+2000,m,d))>=RecDays) then
          begin
            // ÂfBNĝ悤t@C̃XL
            fexsts := False;
            if FindFirst(IncludeTrailingPathDelimiter(IncludeTrailingPathDelimiter(TSFolder)+sr1.Name)+'*.*', faAnyFile, sr2) = 0 then
            begin
              repeat
                if (sr2.Attr and faDirectory) = 0 then
                begin
                  y := StrToIntDef(Copy(sr2.Name,1,2),-1);
                  m := StrToIntDef(Copy(sr2.Name,3,2),-1);
                  d := StrToIntDef(Copy(sr2.Name,5,2),-1);
                  h := StrToIntDef(Copy(sr2.Name,7,2),-1);
                  if ((y<>-1) and (m<>-1) and (d<>-1) and (h<>-1)) then
                  begin
                    if ((nw-EncodeDateTime(y+2000,m,d,h,0,0,0))>RecDays) then
                    begin
                      DeleteFile(IncludeTrailingPathDelimiter(IncludeTrailingPathDelimiter(TSFolder)+sr1.Name)+sr2.Name);
                    end
                      else
                    begin
                      fexsts := True;
                    end;
                  end
                    else
                  begin
                    fexsts := True;
                  end;
                end
                  else
                begin
                  if (sr2.Name<>'.') and (sr2.Name<>'..') then
                  begin
                    fexsts := True;
                  end;
                end;
              until FindNext(sr2) <> 0;
              FindClose(sr2);
            end;
            if (not fexsts) then
            begin
              RemoveDir(IncludeTrailingPathDelimiter(TSFolder)+sr1.Name);
            end;
          end;
        end;
      end;
    until FindNext(sr1) <> 0;
    FindClose(sr1);
  end;
end;


// ==========================================================
//   vpeB
// ==========================================================

procedure TOneSegForm.SetTSFolder(str:String);
var
  INI  : TINIFile;
  str2 : String;
begin
  str2      := ExcludeTrailingPathDelimiter(str);

  FTSFolder := '';
  if (DirectoryExists(str2)) then FTSFolder := str2;

  // INIt@C֏
  INI := TINIFile.Create(ININame);
  try
    INI.WriteString('Config','TSFolder',TSFolder);
  finally
    INI.Free;
  end;

  // \
  if (FTSFolder = '') then
  begin
    TSFolderLabel.Caption := 'ۑFݒ';
  end
    else
  begin
    TSFolderLabel.Caption := 'ۑF'+FTSFolder;
  end;
end;

procedure TOneSegForm.SetRecDays(i:Integer);
var
  INI  : TINIFile;
begin
  if (i>0) then
  begin
    FRecDays := i;
    INI := TINIFile.Create(ININame);
    try
      INI.WriteInteger('Config','RecDays',FRecDays);
    finally
      INI.Free;
    end;
  end;
end;


// ==========================================================
//   `[iID`[iIuWFNg
// ==========================================================

function  TOneSegForm.SerialToTuner(Serial:String):TOneSegTuner;
var
  Tuner : TOneSegTuner;
  i     : Integer;
begin
  Result := nil;
  if Assigned(Tuners) then
  begin
    for i:=0 to Tuners.Count-1 do
    begin
      Tuner := TOneSegTuner(Tuners[i]);
      if Assigned(Tuner) then
      begin
        if (Tuner.TunerID = Serial) then Result := Tuner;
      end;
    end;
  end;
end;


// ==========================================================
//   w`[i
// ==========================================================

procedure TOneSegForm.InitializeInterface(Tuner: TOneSegTuner);
var
  ret: Integer;
begin
  if Assigned(Tuner) then
  begin
    try
      ret := SI_Open(Tuner.Number,Tuner.Handle);
      if (ret <> SI_SUCCESS) then raise Exception.Create('SI_Open('+IntToHex(ret,2)+')');
      ret := SI_FlushBuffers(Tuner.Handle,1,1);
      if (ret <> SI_SUCCESS) then raise Exception.Create('SI_FlushBuffers('+IntToHex(ret,2)+')');
      ret := SI_SetTimeouts(500,500);
      if (ret <> SI_SUCCESS) then raise Exception.Create('SI_SetTimeouts('+IntToHex(ret,2)+')');
      Tuner.StatusText  := '`[i';
      Tuner.TunerState  := STATE_INIT1;
    except
      on E:Exception do
      begin
        if (Tuner.Handle <> nil) then
        begin
          Tuner_Stop(Tuner);
          Sleep(500);
          SI_Close(Tuner.Handle);
        end;
        Tuner.Handle     := nil;
        Tuner.StatusText := 'Error:'+e.Message;
        Tuner.TunerState := STATE_NONE;
      end;
    end;
  end;
end;

procedure TOneSegForm.InitializeTuner(Tuner: TOneSegTuner);
var
  ret: Integer;
begin
  if Assigned(Tuner) then
  begin
    try
      Tuner_Stop(Tuner);
      Sleep(100);
      ret := SI_FlushBuffers(Tuner.Handle,1,1);
      if (ret <> SI_SUCCESS) then raise Exception.Create('SI_FlushBuffers('+IntToHex(ret,2)+')');
      ret := Tuner_Bootup(Tuner);
      if (ret <> SI_SUCCESS) then raise Exception.Create('Bootup('+IntToHex(ret,2)+')');
      if (Tuner.Channel<>-1) then
      begin
        ret := Tuner_SetChannel(Tuner,Tuner.Channel);
          if (ret <> SI_SUCCESS) then raise Exception.Create('SetChannel('+IntToHex(ret,2)+')');
      end;
      Tuner.BeforeBufSize := 0;
      Tuner.StatusText  := '';
      Tuner.LastTime    := Now;
      Tuner.LastTimeDiv := 0;
      Tuner.RetryNum    := 0;
      Tuner.TunerState  := STATE_RUNNING;
    except
      on E:Exception do
      begin
        if (Tuner.Handle <> nil) then
        begin
          Tuner_Stop(Tuner);
          Sleep(100);
          SI_Close(Tuner.Handle);
          Sleep(100);
        end;
        Tuner.Handle     := nil;
        Tuner.StatusText := 'Error:'+e.Message;
        Tuner.TunerState := STATE_NONE;
      end;
    end;
  end;
end;


// ==========================================================
//   w`[i폜
// ==========================================================

procedure TOneSegForm.DeleteTuner(Tuner: TOneSegTuner);
begin
  if Assigned(Tuner) then
  begin
    if (Tuner.Handle<>nil) then
    begin
      Tuner_Stop(Tuner);
      Sleep(500);
      SI_Close(Tuner.Handle);
      Tuner.Handle := nil;
    end;
    if Assigned(Tuner.OutFile) then FreeAndNil(Tuner.OutFile);
    if Assigned(Tuner.IdxFile) then FreeAndNil(Tuner.IdxFile);
  end;
end;


// ==========================================================
//   USBڑ𒲂׃`[iXgXV
// ==========================================================

procedure TOneSegForm.RefreshTunerList;
var
  num  : DWORD;
  ret  : Integer;
  i,n  : Integer;
  pc   : array [0..SI_MAX_DEVICE_STRLEN-1] of Char;
  Tuner: TOneSegTuner;
  INI  : TINIFile;
  flg  : Boolean;
  typ  : Integer;
  psiz : Integer;
  TID  : String;
begin
  INI := TINIFile.Create(ININame);
  flg := False;
  try
    if Assigned(Tuners) then
    begin
      for i:=0 to Tuners.Count-1 do
      begin
        TOneSegTuner(Tuners[i]).CheckFlag := False;
      end;

      ret := SI_GetNumDevices(num);
      if (ret = SI_SUCCESS) then
      begin
        for i:=0 to (num-1) do
        begin
          ret := SI_GetProductString(i,pc,SI_RETURN_DESCRIPTION);
          if (ret = SI_SUCCESS) then
          begin
            // @픻
            typ  := TUNER_NONE;
            if (Pos('UOT-100',String(pc))<>0) then typ := TUNER_UOT;
            if (Pos('PCTV-hiwasa',String(pc))<>0) then
            begin
              if (Pos('Mini',String(pc))<>0) then typ := TUNER_J200 else typ := TUNER_UOT;
            end;
            if (typ = TUNER_UOT) then psiz := 197 else psiz := 196;
            //
            if (typ <> TUNER_NONE) then
            begin
              ret := SI_GetProductString(i,pc,SI_RETURN_SERIAL_NUMBER);
              TID := ANSIReplaceStr(String(pc),'&','_');
              if (ret = SI_SUCCESS) then
              begin
                Tuner := SerialToTuner(TID);
                if (Tuner = nil) then
                begin
                  // New one
                  n := Tuners.Add(TOneSegTuner.Create);
                  Tuner := TOneSegTuner(Tuners[n]);
                  Tuner.TunerID       := TID;
                  Tuner.TunerType     := typ;
                  Tuner.PacketSize    := psiz;
                  Tuner.Handle        := nil;
                  Tuner.Number        := i;
                  Tuner.Channel       := INI.ReadInteger('Tuner'+Tuner.TunerID,'Channel',-1);
                  Tuner.Disable       := INI.ReadBool   ('Tuner'+Tuner.TunerID,'Disable',False);
                  Tuner.FriendlyName  := INI.ReadString ('Tuner'+Tuner.TunerID,'FriendlyName',Tuner.TunerID);
                  Tuner.LastTime      := 0;
                  Tuner.LastTimeDiv   := 0;
                  Tuner.RetryNum      := 0;
                  Tuner.BeforeBufSize := 0;
                  Tuner.CheckFlag     := True;
                  Tuner.WatchDogState := WCHDOG_ENABLE;
                  Tuner.WatchDogSecs  := WCHDOG_SECS;
                  if (Tuner.Disable) then
                  begin
                    Tuner.StatusText    := '';
                    Tuner.TunerState    := STATE_DISABLE;
                  end
                    else
                  begin
                    Tuner.StatusText    := 'I/F';
                    Tuner.TunerState    := STATE_NONE;
                    TunerTab.Tabs.AddObject(Tuner.FriendlyName,Tuner);
                  end;
                  flg := True;
                end
                  else
                begin
                  // Old one
                  Tuner.Number    := i;
                  Tuner.CheckFlag := True;
                end;
              end;
            end;
          end;
        end;

        // Ȃ̂폜
        for i:=Tuners.Count-1 downto 0 do
        begin
          Tuner := TOneSegTuner(Tuners[i]);
          if not Tuner.CheckFlag then
          begin
            for n:=0 to TunerTab.Tabs.Count-1 do
            begin
              if (TOneSegTuner(TunerTab.Tabs.Objects[n]).TunerID = Tuner.TunerID) then
              begin
                TunerTab.Tabs.Delete(n);
                break;
              end;
            end;
            DeleteTuner(Tuner);
            Tuners.Delete(i);
            flg := True;
          end;
        end;
      end;
    end;
  finally
    INI.Free;
  end;

  // ^uXV
  if (flg) then
  begin
    TunerTabChange(TunerTab);
  end
    else
  begin
    TunerTabUpdate(TunerTab);
  end;
end;

// ==========================================================
//   `lꗗXVij
// ==========================================================

procedure TOneSegForm.RefreshChannelList;
var
  i,j,k: Integer;
  str: String;
begin
  ChannelCombo.Items.Clear;
  ChannelCombo.Items.Add('ݒ');
  for i:=13 to 62 do
  begin
    str := IntToStr(i);
    //
    if Assigned(stationID) then
    begin
      k := -1;
      for j:=0 to (stationID.Count-1) do if (stationID.ValueFromIndex[j]=str) then k:=j;
      if (k<>-1) then str := str + ' ('+stationID.Names[k]+')';
    end;
    ChannelCombo.Items.Add(str);
  end;
  
  // ^uXV
  TunerTabChange(TunerTab);
end;


// ==========================================================
//   `[i^u\XV
// ==========================================================

procedure TOneSegForm.TunerTabChange(Sender: TObject);
begin
  if (TunerTab.TabIndex<0) then
  begin
    SelectedTuner := nil;
  end
    else
  begin
    SelectedTuner := TOneSegTuner(TunerTab.Tabs.Objects[TunerTab.TabIndex]);
  end;
  if (SelectedTuner <> nil) then
  begin
    if (SelectedTuner.Channel<0) then
    begin
      if (ChannelCombo.ItemIndex<>0) then ChannelCombo.ItemIndex := 0;
    end
      else
    begin
      if (((SelectedTuner.Channel-13)>=0) and ((SelectedTuner.Channel-13)<(ChannelCombo.Items.Count-1))) then
      begin
        if (ChannelCombo.ItemIndex <> (SelectedTuner.Channel-13+1)) then ChannelCombo.ItemIndex := SelectedTuner.Channel-13+1;
      end;
    end;
  end;
  TunerTabUpdate(Sender);
end;

procedure TOneSegForm.TunerTabUpdate(Sender: TObject);
begin
  if (TunerTab.TabIndex<0) then
  begin
    SelectedTuner := nil;
  end
    else
  begin
    SelectedTuner := TOneSegTuner(TunerTab.Tabs.Objects[TunerTab.TabIndex]);
  end;
  if (SelectedTuner <> nil) then
  begin
    if (TunerIDEdit.Text <> SelectedTuner.TunerID) then TunerIDEdit.Text := SelectedTuner.TunerID;
    if (StatusEdit.Text  <> SelectedTuner.StatusText) then StatusEdit.Text  := SelectedTuner.StatusText;
    if (PacketCountEdit.Text <> IntToStr(SelectedTuner.RecvPacket)) then PacketCountEdit.Text := IntToStr(SelectedTuner.RecvPacket);
    if (TabPanel.Visible<>True) then TabPanel.Visible := True;
  end
    else
  begin
    if (TabPanel.Visible<>False) then TabPanel.Visible := False;
  end;
end;


// ==========================================================
//   j[
// ==========================================================

// I
procedure TOneSegForm.X1Click(Sender: TObject);
begin
  Application.Terminate;
end;

// tH_I
procedure TOneSegForm.R2Click(Sender: TObject);
var
  strDir: String;
begin
  if SelectDirectory(strDir,[],0)  then
  begin
    TSFolder := strDir;
  end;
end;

// nI
procedure TOneSegForm.A1Click(Sender: TObject);
begin
  RefreshChannelList;
end;



// `lύX
procedure TOneSegForm.ChannelComboChange(Sender: TObject);
var
  INI: TINIFile;
begin
  INI := TINIFile.Create(ININame);
  try
    if Assigned(SelectedTuner) then
    begin
      if ((SelectedTuner.Channel <> (ChannelCombo.ItemIndex-1+13)) and (SelectedTuner.TunerState=STATE_RUNNING)) then
      begin
        SelectedTuner.TunerState  := STATE_PENDING;
        Tuner_Stop(SelectedTuner);
        Sleep(200);
        SelectedTuner.Channel := ChannelCombo.ItemIndex-1+13;
        INI.WriteInteger('Tuner'+SelectedTuner.TunerID,'Channel',SelectedTuner.Channel);
        INI.WriteBool   ('Tuner'+SelectedTuner.TunerID,'Disable',SelectedTuner.Disable);
        INI.WriteString ('Tuner'+SelectedTuner.TunerID,'FriendlyName',SelectedTuner.FriendlyName);
        // `lݒ
        SI_FlushBuffers(SelectedTuner.Handle,1,1);
        Tuner_SetChannel(SelectedTuner,SelectedTuner.Channel);
        SelectedTuner.BeforeBufSize := 0;
        SelectedTuner.StatusText    := '`lύX';
        SelectedTuner.RecvPacket    := 0;
        SelectedTuner.LastTime      := Now;
        SelectedTuner.LastTimeDiv   := 0;
        SelectedTuner.RetryNum      := 0;
        SelectedTuner.TunerState    := STATE_RUNNING;
        SelectedTuner.WatchDogState := WCHDOG_ENABLE;
      end;
    end;
  finally
    INI.Free;
  end;
end;


// ==========================================================
//   `[i[ԃ`FbNi3bƁj
// ==========================================================

procedure TOneSegForm.ProbeTimerTimer(Sender: TObject);
var
  i     : Integer;
  Tuner : TOneSegTuner;
  lt    : TDateTime;
begin
  try
    RefreshTunerList;
    if Assigned(Tuners) then
    begin
      for i:=0 to Tuners.Count-1 do
      begin
        Tuner := TOneSegTuner(Tuners[i]);
        if Assigned(Tuner) then
        begin
          if not Tuner.Disable then
          begin
            case Tuner.TunerState of
            STATE_NONE:
              begin
                Tuner.TunerState := STATE_BUSY;
                InitializeInterface(Tuner);
                break;
              end;
            STATE_INIT1:
              begin
                Tuner.TunerState := STATE_BUSY;
                InitializeTuner(Tuner);
                break;
              end;
            STATE_RUNNING:
              begin
                // 
              end;
            end;

            // `[i[~܂ĂH
            if (Tuner.WatchDogState = WCHDOG_ENABLE) then
            begin
              lt := Tuner.LastTime+Tuner.LastTimeDiv;
              if SecondsBetween(lt,Now)>Tuner.WatchDogSecs then
              begin
                Tuner.RetryNum := Tuner.RetryNum+1;
                if (Tuner.RetryNum<5) then
                begin
                  // R}hđM
                  Tuner.TunerState    := STATE_PENDING;
                  Tuner.StatusText    := 'R}hđM';
                  Tuner_Stop(Tuner);
                  Sleep(100);
                  Tuner_SetChannel(Tuner,Tuner.Channel);
                  Tuner.LastTime      := Now;
                  Tuner.LastTimeDiv   := 0;
                  Tuner.TunerState    := STATE_RUNNING;
                  break;
                end
                  else
                begin
                  // Vbg_E
                  DeleteTuner(Tuner);
                  Tuner.StatusText    := 'ď';
                  Tuner.LastTime      := Now;
                  Tuner.LastTimeDiv   := 0;
                  Tuner.TunerState    := STATE_NONE;
                  // PċNWatchDog~߂iI͉xċNȂj
                  Tuner.WatchDogState := WCHDOG_DISABLE;
                  break;
                end;
              end;
            end;
          end;
        end;
      end;
    end;
  except
    //
  end;
end;

procedure TOneSegForm.WindowUpdateTimerTimer(Sender: TObject);
begin
  TunerTabUpdate(Sender);
end;



// ==========================================================
//   `[i[ݒ
// ==========================================================

function TOneSegForm.Tuner_Bootup(Tuner:TOneSegTuner): Integer;
begin
  case Tuner.TunerType of
  TUNER_UOT:  Result := UOT_Bootup(Tuner);
  TUNER_J200: Result := J200_Bootup(Tuner);
  else        Result := SI_INVALID_PARAMETER;
  end;
end;

function TOneSegForm.Tuner_SetChannel(Tuner:TOneSegTuner; ch:Integer): Integer;
begin
  case Tuner.TunerType of
  TUNER_UOT:  Result := UOT_SetChannel(Tuner,ch);
  TUNER_J200: Result := J200_SetChannel(Tuner,ch);
  else        Result := SI_INVALID_PARAMETER;
  end;
end;

function TOneSegForm.Tuner_Stop(Tuner:TOneSegTuner): Integer;
begin
  case Tuner.TunerType of
  TUNER_UOT:  Result := UOT_Stop(Tuner);
  TUNER_J200: Result := J200_Stop(Tuner);
  else        Result := SI_INVALID_PARAMETER;
  end;
end;


function TOneSegForm.UOTJ200_PlainSend(Tuner:TOneSegTuner; reg1:BYTE; reg2:BYTE; reg3:BYTE; reg4:BYTE; reg5:BYTE): Integer;
var
  ret : Integer;
  num : DWORD;
  buf : array [0..4] of Char;
begin
  ret := SI_WRITE_ERROR;
  if (Integer(Tuner.Handle) >= 0) then
  begin
    buf[0] := Chr(reg1);
    buf[1] := Chr(reg2);
    buf[2] := Chr(reg3);
    buf[3] := Chr(reg4);
    buf[4] := Chr(reg5);
    ret := SI_Write(Tuner.Handle,buf,5,num,nil);
    if (num <> 5) then ret := SI_WRITE_ERROR;
  end;
  Result := ret;
end;


function TOneSegForm.UOTJ200_SetRegister(Tuner:TOneSegTuner; reg:WORD; dat:BYTE): Integer;
var
  ret : Integer;
  num : DWORD;
  buf : array [0..4] of Char;
begin
  ret := SI_WRITE_ERROR;
  if (Integer(Tuner.Handle) >= 0) then
  begin
    buf[0] := #$00;
    buf[1] := Chr(reg shr 8);
    buf[2] := Chr(reg and $FF);
    buf[3] := Chr(dat);
    buf[4] := #$00;
    ret := SI_Write(Tuner.Handle,buf,5,num,nil);
    if (num <> 5) then ret := SI_WRITE_ERROR;
  end;
  Sleep(7);
  Result := ret;
end;

function TOneSegForm.UOTJ200_ReadRegister(Tuner:TOneSegTuner; reg:WORD; var dat:BYTE): Integer;
var
  ret : Integer;
  num : DWORD;
  len : DWORD;
  buf : array [0..19] of Char;
begin
  ret := SI_WRITE_ERROR;
  if (Integer(Tuner.Handle) >= 0) then
  begin
    // Write
    buf[0] := #$01;
    buf[1] := Chr(reg shr 8);
    buf[2] := Chr(reg and $FF);
    buf[3] := #$FF;
    buf[4] := #$FF;
    ret := SI_Write(Tuner.Handle,buf,5,num,nil);
    if (num <> 5) then ret := SI_WRITE_ERROR;
    Sleep(10);

    if (ret = SI_SUCCESS) then
    begin
      // Read
      len := CNF_SEARCHPKT;
      while(len>0) do
      begin
        ret := SI_Read(Tuner.Handle,buf,1,num,nil);
        if (ret <> SI_SUCCESS) then begin Result := ret; exit; end;
        len := len - 1;
        if (buf[0] = #$48) then break;
      end;
      ret := SI_Read(Tuner.Handle,buf,19,num,nil);
      if (num <> 19) then
      begin
        Result := SI_READ_ERROR;
        exit;
      end;

      if ((buf[0]<>#$75) or (buf[1]<>#$09) or (buf[2]<>#$15) or
          (buf[3]<>#$58) or (buf[4]<>#$01) or (buf[5]<>#$01)) then
      begin
        Result := SI_READ_ERROR;
        exit;
      end;
      dat := Ord(buf[6]);
    end;
  end;
  Sleep(50);
  Result := ret;
end;


// ==========================================================
//   `[i[ݒ (UOT-100 / LOG-J100)
// ==========================================================

function TOneSegForm.UOT_Bootup(Tuner:TOneSegTuner): Integer;
const
  regset1:Array [$173 .. $1b6] of BYTE = (
      $00,$70,$89,$c3,$12,$83,$00,$20,$00,$27,$14,$04,$b0,$00,$93,$30,
      $09,$1d,$8c,$9c,$a4,$09,$ff,$b2,$94,$ce,$55,$3a,$7f,$81,$7f,$81,
      $01,$c6,$72,$46,$7e,$13,$00,$00,$00,$00,$00,$e0,$7f,$04,$b0,$7e,
      $7f,$02,$1f,$1f,$1f,$1f,$1f,$1f,$1f,$1f,$1f,$1f,$1f,$1f,$1f,$1f,
      $1f,$1f,$21,$02 );
  regset2: Array [0..25,0..1] of WORD = (
      ($17F, $B0), ($174, $80), ($001, $01), ($005, $01), ($006, $70),
      ($04D, $0E), ($070, $01), ($040, $0F), ($043, $F8), ($047, $C0),
      ($0A1, $40), ($0A5, $4C), ($0B9, $D9), ($0AC, $01), ($0AD, $80),
      ($0AE, $43), ($0AF, $01), ($0B0, $22), ($0BD, $E9), ($0C0, $1C),
      ($0C4, $20), ($0C5, $04), ($0CA, $33), ($008, $0C), ($009, $12),
      ($001, $02)
  );
  regset3: Array [0..5,0..1] of WORD = (
      ($175, $89), ($178, $83), ($179, $00), ($17A, $20), ($17B, $00),
      ($17D, $14)
  );
var
  dat : BYTE;
  i   : Integer;
begin
  // First bootup
  for i:=$173 to $1b6 do
  begin
    Result := UOTJ200_SetRegister(Tuner,i,regset1[i]);     if (Result <> SI_SUCCESS) then exit;
  end;

  Result := UOTJ200_SetRegister  (Tuner,$1A2,$00);         if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$1A3,$48);         if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_ReadRegister (Tuner,$17F,dat);         if (Result <> SI_SUCCESS) then exit;

  Result := UOTJ200_SetRegister  (Tuner,$17F,$70);         if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_ReadRegister (Tuner,$168,dat);         if (Result <> SI_SUCCESS) then exit;

  for i:=0 to 25 do
  begin
    Result := UOTJ200_SetRegister(Tuner,regset2[i][0],regset2[i][1]); if (Result <> SI_SUCCESS) then exit;
  end;
  Sleep(90);
  for i:=0 to 5 do
  begin
    Result := UOTJ200_SetRegister(Tuner,regset3[i][0],regset3[i][1]); if (Result <> SI_SUCCESS) then exit;
  end;
end;

function TOneSegForm.UOT_SetChannel(Tuner:TOneSegTuner; ch:Integer): Integer;
const
//     0   1   2   3   4   5   6   7   8   9
  p1  : Array [13 .. 62] of BYTE = (
                  $88,$89,$89,$8A,$8A,$8A,$8B,
      $8B,$8C,$8C,$8D,$8D,$8D,$8E,$8E,$8F,$8F,
      $86,$86,$87,$87,$88,$88,$89,$89,$8A,$8A,
      $8A,$8B,$8B,$8C,$8C,$8C,$8D,$8D,$8D,$8E,
      $8E,$8F,$8F,$86,$86,$86,$87,$87,$87,$88,
      $88,$89,$8A );
  p2  : Array [13 .. 62] of BYTE = (
                  $30,$30,$30,$30,$30,$30,$30,
      $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,
      $70,$70,$70,$70,$70,$70,$70,$70,$70,$70,
      $70,$70,$70,$70,$70,$70,$70,$70,$70,$70,
      $70,$70,$70,$B0,$B0,$B0,$B0,$B0,$B0,$B0,
      $B0,$B0,$B0 );
var
  dat,tune : BYTE;
  p1a,p2a  : BYTE;
  p2bef    : BYTE;
  p3,p4,p5 : BYTE;
  mhz      : Integer;
  flg      : Boolean;
  count    : Integer;
begin
  if ((ch>=13) and (ch<=62)) then
  begin
    // f[^Zbg
    mhz   := 473 + (ch-13)*6;
    p3    := (mhz shr 4);
    p4    := (mhz and $0F);
    if (ch<=20) then p5 := $8c else p5 := $94;
    p1a   := p1[ch];
    p2a   := p2[ch];
    p2bef := 0;
    flg   := False;
    count := 10;
    //
    while ((not flg) and (count>0)) do
    begin
      if (p2bef <> p2a) then
      begin
        Result := UOTJ200_SetRegister  (Tuner,$178,$83);     if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$175,p1a);     if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$182,p2a);     if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$184,p3);      if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$185,p5);      if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$186,$9C);     if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$187,$A4);     if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$188,p4);      if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$183,$0B);     if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$183,$09);     if (Result <> SI_SUCCESS) then exit;
        p2bef  := p2a;
      end
        else
      begin
        Result := UOTJ200_SetRegister  (Tuner,$175,p1a);     if (Result <> SI_SUCCESS) then exit;
        Result := UOTJ200_SetRegister  (Tuner,$182,p2a);     if (Result <> SI_SUCCESS) then exit;
        p2bef  := p2a;
      end;
      Result := UOTJ200_ReadRegister (Tuner,$170,dat);       if (Result <> SI_SUCCESS) then exit;
      //
      tune := (dat shr 4) and $0F;
      if (tune<3) then
      begin
        // Step back
        p1a := p1a - 1;
        if (p1a<$86) then
        begin
          p2a := ((((p2a shr 5)-1) shl 5) or $10);
        end;
      end
        else
      begin
        if (tune>$C) then
        begin
          // Step forward
          p1a := p1a + 1;
          if (p1a>$8F) then
          begin
            p2a := ((((p2a shr 5)+1) shl 5) or $10);
          end;
        end
          else
        begin
          // Tune OK!
          flg := True;
        end;
      end;
      count := count - 1;
    end;
    Result := UOTJ200_ReadRegister (Tuner,$169,dat);         if (Result <> SI_SUCCESS) then exit;
    //
    Result := UOTJ200_SetRegister  (Tuner,$001,$02);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_ReadRegister (Tuner,$032,dat);         if (Result <> SI_SUCCESS) then exit;
    if ((dat and $08)<>0) then
    begin
      // MOK!
      Result := UOTJ200_PlainSend(Tuner,$02,$00,$00,$00,$00);
    end
      else
    begin
      // Ms(dg݂炸)
      // ߂̂V[PXǁAƂ肠X^[gĂ݂
      Result := UOTJ200_PlainSend(Tuner,$02,$00,$00,$00,$00);
    end;
  end
    else
  begin
    Result := SI_INVALID_PARAMETER;
  end;
end;

function TOneSegForm.UOT_Stop(Tuner:TOneSegTuner): Integer;
begin
  // Stop
  Result := UOTJ200_PlainSend(Tuner,$03,$00,$00,$00,$00);
end;




// ==========================================================
//   `[i[ݒ (LOG-J200)
// ==========================================================


function TOneSegForm.J200_Initialize(Tuner:TOneSegTuner; var dat:BYTE): Integer;
var
  ret : Integer;
  num : DWORD;
  len : DWORD;
  buf : array [0..19] of Char;
begin
  ret := SI_WRITE_ERROR;
  if (Integer(Tuner.Handle) >= 0) then
  begin
    // Write
    buf[0] := #$06;
    buf[1] := #$00;
    buf[2] := #$00;
    buf[3] := #$00;
    buf[4] := #$00;
    ret := SI_Write(Tuner.Handle,buf,5,num,nil);
    if (num <> 5) then ret := SI_WRITE_ERROR;

    if (ret = SI_SUCCESS) then
    begin
      // Read
      len := CNF_SEARCHPKT;
      while(len>0) do
      begin
        ret := SI_Read(Tuner.Handle,buf,1,num,nil);
        if (ret <> SI_SUCCESS) then begin Result := ret; exit; end;
        len := len - 1;
        if (buf[0] = #$48) then break;
      end;
      ret := SI_Read(Tuner.Handle,buf,19,num,nil);
      if (num <> 19) then
      begin
        Result := SI_READ_ERROR;
        exit;
      end;

      if ((buf[0]<>#$75) or (buf[1]<>#$09) or (buf[2]<>#$15) or
          (buf[3]<>#$58) or (buf[4]<>#$06) or (buf[5]<>#$06)) then
      begin
        Result := SI_READ_ERROR;
        exit;
      end;
      dat := Ord(buf[6]);
    end;
  end;
  Result := ret;
end;


function TOneSegForm.J200_Bootup(Tuner:TOneSegTuner): Integer;
const
  regset1: Array [0..163,0..1] of WORD = (
      ($001, $40), ($002, $00), ($004, $00), ($005, $00), ($006, $00),
      ($007, $00), ($008, $00), ($00C, $00), ($00F, $0A), ($013, $1B),
      ($014, $80), ($015, $40), ($017, $70), ($018, $01), ($019, $13),
      ($01A, $01), ($01B, $13), ($01C, $80), ($01D, $00), ($01E, $02),
      ($01F, $08), ($020, $00), ($021, $00), ($022, $80), ($023, $4C),
      ($024, $4C), ($025, $00), ($026, $00), ($028, $60), ($02B, $40),
      ($02C, $FF), ($02D, $00), ($02E, $FF), ($02F, $00), ($030, $28),
      ($031, $07), ($032, $1B), ($034, $0F), ($039, $63), ($03A, $10),
      ($03B, $30), ($03C, $00), ($03F, $18), ($040, $0C), ($041, $00),
      ($042, $00), ($043, $7F), ($044, $FF), ($045, $40), ($047, $00),
      ($048, $FF), ($049, $E6), ($04B, $54), ($04C, $03), ($04F, $02),
      ($050, $80), ($052, $20), ($054, $7A), ($055, $7B), ($056, $40),
      ($057, $70), ($058, $FF), ($05C, $50), ($05D, $00), ($05E, $00),
      ($060, $20), ($070, $18), ($071, $30), ($075, $08), ($076, $00),
      ($077, $03), ($0B0, $00), ($0B1, $04), ($0B2, $7C), ($0B3, $21),
      ($0B4, $0F), ($0B5, $FF), ($0B6, $FF), ($0B7, $FF), ($0B8, $FF),
      ($0B9, $FC), ($0BA, $00), ($0BB, $00), ($0BC, $00), ($0C0, $A3),
      ($0C1, $01), ($0D0, $20), ($0D7, $CC), ($0E3, $17), ($0E4, $82),
      ($0EA, $1F), ($0EC, $30), ($0ED, $50), ($0EE, $38), ($0F0, $00),
      ($0F1, $00), ($0F2, $00), ($0F3, $00), ($0F4, $00), ($0F5, $00),
      ($0F6, $00), ($0F6, $00), ($173, $00), ($174, $70), ($176, $C3),
      ($177, $24), ($179, $00), ($17A, $20), ($17B, $00), ($17C, $22),
      ($17D, $14), ($17E, $06), ($17F, $90), ($180, $00), ($181, $93),
      ($183, $09), ($189, $FF), ($18A, $B2), ($18B, $94), ($18C, $CE),
      ($18D, $55), ($18E, $3A), ($18F, $7F), ($190, $81), ($191, $7F),
      ($192, $81), ($193, $01), ($194, $C6), ($195, $72), ($196, $46),
      ($197, $7E), ($198, $13), ($199, $00), ($19A, $00), ($19B, $00),
      ($19C, $00), ($19D, $00), ($19E, $C0), ($19F, $7F), ($1A0, $05),
      ($1A1, $B0), ($1A2, $7E), ($1A3, $7F), ($1A4, $00), ($1A5, $1F),
      ($1A6, $1F), ($1A7, $1F), ($1A8, $1F), ($1A9, $1F), ($1AA, $1F),
      ($1AB, $1F), ($1AC, $1F), ($1AD, $1F), ($1AE, $1F), ($1AF, $1F),
      ($1B0, $1F), ($1B1, $1F), ($1B2, $1F), ($1B3, $1F), ($1B4, $1F),
      ($1B5, $21), ($1B6, $02), ($1A2, $00), ($1A3, $48)
  );
var
  i  : Integer;
  dat: BYTE;
begin
  // First bootup
  Result := J200_Initialize(Tuner,dat);                      if (Result <> SI_SUCCESS) then exit;
  //
  for i:=0 to 163 do
  begin
    Result := UOTJ200_SetRegister(Tuner,regset1[i][0],regset1[i][1]); if (Result <> SI_SUCCESS) then exit;
  end;

  Result := UOTJ200_SetRegister  (Tuner,$178,$83);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$175,$89);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$182,$30);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$184,$1A);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$185,$8C);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$186,$55);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$187,$55);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$188,$09);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$183,$0B);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$183,$09);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_ReadRegister (Tuner,$170,dat);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_ReadRegister (Tuner,$17F,dat);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$17F,$50);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_ReadRegister (Tuner,$168,dat);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$17F,$90);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$174,$80);           if (Result <> SI_SUCCESS) then exit;
end;

function TOneSegForm.J200_SetChannel(Tuner:TOneSegTuner; ch:Integer): Integer;
const
//     0   1   2   3   4   5   6   7   8   9
  p1  : Array [13 .. 62] of BYTE = (
                  $88,$89,$89,$8A,$8A,$8A,$8B,
      $8B,$8C,$8C,$8D,$8D,$8D,$8E,$8E,$8F,$8F,
      $86,$86,$87,$87,$88,$88,$89,$89,$8A,$8A,
      $8A,$8B,$8B,$8C,$8C,$8C,$8D,$8D,$8D,$8E,
      $8E,$8F,$8F,$86,$86,$86,$87,$87,$87,$88,
      $88,$89,$8A );
  p2  : Array [13 .. 62] of BYTE = (
                  $30,$30,$30,$30,$30,$30,$30,
      $30,$30,$30,$30,$30,$30,$30,$30,$30,$30,
      $70,$70,$70,$70,$70,$70,$70,$70,$70,$70,
      $70,$70,$70,$70,$70,$70,$70,$70,$70,$70,
      $70,$70,$70,$B0,$B0,$B0,$B0,$B0,$B0,$B0,
      $B0,$B0,$B0 );
var
  dat: BYTE;
  byt: Array [0..6] of BYTE;
begin
  // Block 1 (Ch21Ch40͏o͂ȂB̃\tg̃oOۂC邯ǁcj
  if (ch<=20) then
  begin
    Result := UOTJ200_SetRegister  (Tuner,$173,$00);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$176,$C3);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$177,$24);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$178,$83);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17A,$70); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17B,$80); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17C,$77); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17D,$94); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$195,$72);         if (Result <> SI_SUCCESS) then exit;
  end;
  if (ch>=22) and (ch<=39) then
  begin
    Result := UOTJ200_SetRegister  (Tuner,$173,$00);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$176,$C3);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$177,$24);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$178,$83);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17A,$73); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17B,$C6); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17C,$77); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17D,$DC); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$195,$72);         if (Result <> SI_SUCCESS) then exit;
  end;
  if (ch>=41) then
  begin
    Result := UOTJ200_SetRegister  (Tuner,$173,$00);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$176,$C3);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$177,$24);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$178,$83);         if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17A,$FB); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17B,$C6); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17C,$FF); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$17D,$DC); { <- }  if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$195,$72);         if (Result <> SI_SUCCESS) then exit;
  end;

  // Block 2
  byt[0] := p1[ch];
  byt[1] := p2[ch];
  byt[2] := $1A + ((ch - 13) div 3);
  if (ch<=20) then byt[3] := $8C else byt[3] := $94;
  case ((ch-13) mod 3) of
  0:
    begin
      byt[4] := $18;
      byt[5] := $04;
      byt[6] := $05;
    end;
  1:
    begin
      byt[4] := $6E;
      byt[5] := $59;
      byt[6] := $0A;
    end;
  else
    begin
      byt[4] := $C3;
      byt[5] := $AE;
      byt[6] := $0F;
    end;
  end;
  Result := UOTJ200_SetRegister  (Tuner,$178,$83);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$175,byt[0]);        if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$182,byt[1]);        if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$184,byt[2]);        if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$185,byt[3]);        if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$186,byt[4]);        if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$187,byt[5]);        if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$188,byt[6]);        if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$183,$0B);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$183,$09);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_ReadRegister (Tuner,$170,dat);           if (Result <> SI_SUCCESS) then exit;
  if ((dat and $F0)=$20) then
  begin
    // block3
    byt[0] := byt[0]-1;
    if (byt[0]<$86) then
    begin
      byt[1] := ((((byt[1] shr 5)-1) shl 5) or $10);
    end;
    Result := UOTJ200_SetRegister  (Tuner,$175,byt[0]);      if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_SetRegister  (Tuner,$182,byt[1]);      if (Result <> SI_SUCCESS) then exit;
    Result := UOTJ200_ReadRegister (Tuner,$170,dat);         if (Result <> SI_SUCCESS) then exit;
  end;

  // nego
  Result := UOTJ200_ReadRegister (Tuner,$169,dat);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$047,$80);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_SetRegister  (Tuner,$001,$40);           if (Result <> SI_SUCCESS) then exit;
  Result := UOTJ200_ReadRegister (Tuner,$080,dat);           if (Result <> SI_SUCCESS) then exit;
  if ((dat and $F0)=$00) then
  begin
    // MOK!
    Result := UOTJ200_ReadRegister (Tuner,$0B0,dat);         if (Result <> SI_SUCCESS) then exit;
    // Start
    Result := UOTJ200_PlainSend(Tuner,$02,$00,$00,$00,$00);
  end
    else
  begin
    // Retry
    Result := UOTJ200_ReadRegister (Tuner,$0B0,dat);         if (Result <> SI_SUCCESS) then exit;
    if ((dat and $01)<>0) then
    begin
      Result := UOTJ200_SetRegister  (Tuner,$001,$40);       if (Result <> SI_SUCCESS) then exit;
    end;
    Result := UOTJ200_ReadRegister (Tuner,$080,dat);         if (Result <> SI_SUCCESS) then exit;
    if ((dat and $F0)=$00) then
    begin
      // MOK!
      Result := UOTJ200_ReadRegister (Tuner,$0B0,dat);       if (Result <> SI_SUCCESS) then exit;
      // Start
      Result := UOTJ200_PlainSend(Tuner,$02,$00,$00,$00,$00);
    end
      else
    begin
      // Ms(dg݂炸)
      // ߂̂V[PXǁAƂ肠X^[gĂ݂
      Result := UOTJ200_ReadRegister (Tuner,$0B0,dat);       if (Result <> SI_SUCCESS) then exit;
      // Start
      Result := UOTJ200_PlainSend(Tuner,$02,$00,$00,$00,$00);
    end;
  end;
end;

function TOneSegForm.J200_Stop(Tuner:TOneSegTuner): Integer;
begin
  // Stop
  Result := UOTJ200_PlainSend(Tuner,$03,$00,$00,$00,$00);
end;





// ==========================================================
//   ^`FbNi100msƁj
// ==========================================================

procedure TOneSegForm.RecordTimerTimer(Sender: TObject);
var
  i         : Integer;
  Tuner     : TOneSegTuner;
  RxLen,Stat: DWORD;
  ret       : Integer;
  buf       : array [0..SI_MAX_READ_SIZE] of Char;
  totcode   : array [0..4] of Char;
  num       : DWORD;
  pos       : Integer;
  tot       : TDateTime;
  str,datstr: String;
  fn        : String;
  ptr       : DWORD;
begin
  try
    if Assigned(Tuners) then
    begin
      for i:=0 to Tuners.Count-1 do
      begin
        Tuner := TOneSegTuner(Tuners[i]);

        if (Tuner <> nil) then
        begin
          if (Tuner.TunerState = STATE_RUNNING) and (not Tuner.Disable) then
          begin
            ret := SI_CheckRXQueue(Tuner.Handle, RxLen, Stat);
            if (ret = SI_SUCCESS) then
            begin
              while (RxLen >= Tuner.PacketSize) do
              begin
                // MI
                if (RxLen>SI_MAX_READ_SIZE) then RxLen := SI_MAX_READ_SIZE;
                Move(Tuner.BeforeBuf,buf,Tuner.BeforeBufSize);
                ret := SI_Read(Tuner.Handle,buf[Tuner.BeforeBufSize],RxLen,num,nil);
                num := num + DWORD(Tuner.BeforeBufSize);
                Tuner.BeforeBufSize := 0;
                pos := 0;

                if (ret = SI_SUCCESS) then
                begin
                  while ((buf[pos+0] <> #$47) and (num >= Tuner.PacketSize)) do
                  begin
                    pos := pos + 1;
                    num := num - 1;
                  end;

                  while(num >= Tuner.PacketSize) do
                  begin
                    if (buf[pos+0] = #$47) then
                    begin
                      // TOTpPbg?
                      if (((ord(buf[pos+1]) and $1F) = $00) and (buf[pos+2] = #$14)) then
                      begin
                        // TOTpPbg
                        tot := AnalyzeTOT(@buf[pos],totcode);
                        if (tot <> 0) then
                        begin
                          Tuner.LastTime := tot;
                          Tuner.LastTimeDiv := Now-tot;
                          DateTimeToString(str,'yy/mm/dd hh:nn:ss',Tuner.LastTime);
                          Tuner.StatusText := 'MF'+str;
                          Tuner.WatchDogState := WCHDOG_ENABLE;       // f[^MWatchDog܂Lɂ
                          //
                          DateTimeToString(str   ,'yymmddhh',Tuner.LastTime);
                          DateTimeToString(datstr,'yymmdd'  ,Tuner.LastTime);
                          Tuner.FileName := str+'_Ch'+IntToStr(Tuner.Channel);
                          Tuner.FilePath := datstr;
                          //
                          if (Tuner.FileName <> Tuner.BeforeFileName) then
                          begin
                            // Close before files
                            if Assigned(Tuner.OutFile) then FreeAndNil(Tuner.OutFile);
                            if Assigned(Tuner.IdxFile) then FreeAndNil(Tuner.IdxFile);

                            // Directory
                            if (not DirectoryExists(TSFolder+'\'+Tuner.FilePath)) then
                            begin
                              MkDir(TSFolder+'\'+Tuner.FilePath);
                            end;

                            // Open New file
                            fn := TSFolder+'\'+Tuner.FilePath+'\'+Tuner.FileName;
                            if (FileExists(fn+'.ts')) then
                            begin
                              Tuner.OutFile := TFileStream.Create(fn+'.ts', fmOpenReadWrite or fmShareDenyNone);
                            end
                              else
                            begin
                              Tuner.OutFile := TFileStream.Create(fn+'.ts', fmCreate or fmShareDenyNone);
                              Tuner.OutFile.Free;
                              Tuner.OutFile := TFileStream.Create(fn+'.ts', fmOpenReadWrite or fmShareDenyNone);
                            end;
                            Tuner.OutFile.Seek(0,soFromEnd);

                            // Open New index file
                            if (FileExists(fn+'.idx')) then
                            begin
                              Tuner.IdxFile := TFileStream.Create(fn+'.idx', fmOpenReadWrite or fmShareDenyNone);
                            end
                              else
                            begin
                              Tuner.IdxFile := TFileStream.Create(fn+'.idx', fmCreate or fmShareDenyNone);
                              Tuner.IdxFile.Free;
                              Tuner.IdxFile := TFileStream.Create(fn+'.idx', fmOpenReadWrite or fmShareDenyNone);
                            end;
                            Tuner.IdxFile.Seek(0,soFromEnd);

                            // New file stand-by
                            Tuner.BeforeFileName := Tuner.FileName;
                          end;

                          // index
                          if Assigned(Tuner.IdxFile) then
                          begin
                            ptr := Tuner.OutFile.Position;
                            Tuner.IdxFile.Write(totcode,5);
                            Tuner.IdxFile.Write(ptr,sizeof(ptr));
                          end;

                          // svt@C̍폜
                          if ((HourOf(tot) <> HourOf(LastDeleteNow)) and (tot>LastDeleteNow)) then
                          begin
                            DeleteOldFiles(tot);
                            LastDeleteNow := tot;
                          end;
                        end;
                      end;
                      if Assigned(Tuner.OutFile) then Tuner.OutFile.Write(buf[pos],188);
                      Tuner.RecvPacket := Tuner.RecvPacket + 1;
                    end;
                    pos := pos + Tuner.PacketSize;
                    num := num - Tuner.PacketSize;
                  end;
                  Tuner.BeforeBufSize := num;
                  Move(buf[pos],Tuner.BeforeBuf,num);
                end;

                ret := SI_CheckRXQueue(Tuner.Handle, RxLen, Stat);
                if (ret <> SI_SUCCESS) then RxLen := 0;
              end;
            end
              else
            begin
              // USBG[`[i[؂藣
              SI_Close(Tuner.Handle);
              Tuner.Handle := nil;
              Tuner.TunerState := STATE_NONE;
              Tuner.StatusText := '`[i[~';
            end;
          end;
        end;
     end;
    end;
  except
    //
  end;
end;

// ----------------------------
//   TOTpPbg
// ----------------------------

function TOneSegForm.AnalyzeTOT(buf:PChar; tot:PChar): TDateTime;
var
  ptr,ptr2,i  : Integer;
  table_id    : BYTE;
  sect_len    : WORD;
  mjd         : Integer;
  h,m,s       : Integer;
  ch          : BYTE;
begin
  Result := 0;
  //
  ptr := 5;
  //
  repeat
    table_id := ord(buf[ptr]);
    ptr := ptr + 1;
    if (table_id <> $FF) then
    begin
      sect_len := (ord(buf[ptr])*256 + ord(buf[ptr+1])) and $0FFF;
      ptr  := ptr + 2;
      ptr2 := ptr;

      case table_id of
      $73:
        begin
          // TOT
          mjd := (ord(buf[ptr2])*256 + ord(buf[ptr2+1]));
          ch := ord(buf[ptr2+2]); h := (ch shr 4)*10 + (ch and $F);
          ch := ord(buf[ptr2+3]); m := (ch shr 4)*10 + (ch and $F);
          ch := ord(buf[ptr2+4]); s := (ch shr 4)*10 + (ch and $F);
          Result := ModifiedJulianDateToDateTime(mjd) + EncodeTime(h,m,s,0);   //
          for i:=0 to 4 do tot[i] := buf[ptr2+i];
        end;
      end;
      ptr := ptr + sect_len;
    end;
  until ((ptr>=188) or (table_id = $FF));
end;

procedure TOneSegForm.D1Click(Sender: TObject);
var
  cnt,i:Integer;
begin
  if Assigned(Tuners) then
  begin
    cnt := 0;
    for i:=0 to Tuners.Count-1 do if not TOneSegTuner(Tuners[i]).Disable then cnt := cnt + 1;
    DaysForm.Channels := cnt;
    DaysForm.Days := RecDays;
    if (DaysForm.ShowModal = mrOK) then
    begin
      RecDays := DaysForm.Days;
    end;
  end;
end;

procedure TOneSegForm.N1Click(Sender: TObject);
begin
  Application.Restore;
end;


end.
